@@ -1,7 +1,8 @@
 from functools import lru_cache, reduce
 from typing import List, Union, TextIO, Optional
 import deprecation
-from linkml_model.meta import SchemaDefinition, Element, Definition, ClassDefinition, SlotDefinition
+from linkml_runtime.linkml_model.meta import SchemaDefinition, Element, ElementName, \
+    Definition, ClassDefinition, SlotDefinition
 
 from bmt.toolkit_generator import ToolkitGenerator
 from bmt.utils import format_element, parse_name
@@ -9,8 +10,8 @@ from bmt.utils import format_element, parse_name
 Url = str
 Path = str
 
-REMOTE_PATH = 'https://raw.githubusercontent.com/biolink/biolink-model/1.8.0/biolink-model.yaml'
-
+REMOTE_PATH = 'https://raw.githubusercontent.com/biolink/biolink-model/2.2.0/biolink-model.yaml'
+RELATED_TO = 'related to'
 CACHE_SIZE = 1024
 
 
@@ -249,7 +250,10 @@ class Toolkit(object):
         return filtered_elements
 
     @lru_cache(CACHE_SIZE)
-    def get_ancestors(self, name: str, reflexive: bool = True, formatted: bool = False) -> List[str]:
+    def get_ancestors(self, name: str,
+                      reflexive: bool = True,
+                      formatted: bool = False,
+                      mixin: bool = True) -> List[str]:
         """
         Gets a list of names of ancestors.
 
@@ -261,6 +265,8 @@ class Toolkit(object):
             Whether to include the query element in the list of ancestors
         formatted: bool
             Whether to format element names as CURIEs
+        mixin: bool
+            If True, then that means we want to find mixin ancestors as well as is_a ancestors
 
         Returns
         -------
@@ -272,6 +278,9 @@ class Toolkit(object):
         ancs = []
         if isinstance(element, (ClassDefinition, SlotDefinition)):
             a = self.generator.ancestors(element)
+            if mixin:
+                mixins_parents = self._get_mixin_descendants(a)
+                a = a + mixins_parents
             ancs = a if reflexive else a[1:]
         if isinstance(element, SlotDefinition):
             filtered_ancs = self._filter_secondary(ancs)
@@ -279,8 +288,22 @@ class Toolkit(object):
             filtered_ancs = ancs
         return self._format_all_elements(filtered_ancs, formatted)
 
+    def _get_mixin_descendants(self, ancestors: List[ElementName]) -> List[ElementName]:
+        mixins_parents = []
+        for ancestor in ancestors:
+            a_element = self.get_element(ancestor)
+            if a_element.mixins:
+                for mixin in a_element.mixins:
+                    mixin_element = self.get_element(mixin)
+                    mixin_parents = self.generator.ancestors(mixin_element)
+                    mixins_parents = mixins_parents + mixin_parents
+        return mixins_parents
+
     @lru_cache(CACHE_SIZE)
-    def get_descendants(self, name: str, reflexive: bool = True, formatted: bool = False) -> List[str]:
+    def get_descendants(self, name: str,
+                        reflexive: bool = True,
+                        formatted: bool = False,
+                        mixin: bool = True) -> List[str]:
         """
         Gets a list of names of descendants.
 
@@ -292,6 +315,8 @@ class Toolkit(object):
             Whether to include the query element in the list of ancestors
         formatted: bool
             Whether to format element names as CURIEs
+        mixin: bool
+            If True, then that means we want to find mixin ancestors as well as is_a ancestors
 
         Returns
         -------
@@ -302,8 +327,9 @@ class Toolkit(object):
         desc = []
         filtered_desc = []
         element = self.get_element(name)
+
         if element:
-            d = self.generator.descendants(element.name)
+            d = self.generator.descendants(element.name, mixin)
             if reflexive:
                 desc.append(element.name)
             desc += d
@@ -314,7 +340,7 @@ class Toolkit(object):
         return self._format_all_elements(filtered_desc, formatted)
 
     @lru_cache(CACHE_SIZE)
-    def get_children(self, name: str, formatted: bool = False) -> List[str]:
+    def get_children(self, name: str, formatted: bool = False, mixin: bool = True) -> List[str]:
         """
         Gets a list of names of children.
 
@@ -324,6 +350,8 @@ class Toolkit(object):
             The name of an element in the Biolink Model
         formatted: bool
             Whether to format element names as CURIEs
+        mixin: bool
+            If True, then that means we want to find mixin ancestors as well as is_a ancestors
 
         Returns
         -------
@@ -334,7 +362,7 @@ class Toolkit(object):
         children = []
         element = self.get_element(name)
         if element:
-            children = self.generator.children(element.name)
+            children = self.generator.children(element.name, mixin)
         return self._format_all_elements(children, formatted)
 
     @lru_cache(CACHE_SIZE)
@@ -392,7 +420,11 @@ class Toolkit(object):
                 element = self.get_element(name.replace('_', ' '))
         return element
 
-    def get_slot_domain(self, slot_name, include_ancestors: bool = False, formatted: bool = False) -> List[str]:
+    def get_slot_domain(self,
+                        slot_name,
+                        include_ancestors: bool = False,
+                        formatted: bool = False,
+                        mixin: bool = True) -> List[str]:
         """
         Get the domain for a given slot.
 
@@ -404,6 +436,8 @@ class Toolkit(object):
             Whether or not to include ancestors of the domain class
         formatted: bool
             Whether to format element names as CURIEs
+        mixin: bool
+            If True, then that means we want to find mixin ancestors as well as is_a ancestors
 
         Returns
         -------
@@ -414,22 +448,24 @@ class Toolkit(object):
         slot_domain = []
         domain_classes = set()
         element = self.get_element(slot_name)
-        if element:
-            if element.domain:
-                domain_classes.add(element.domain)
+        if element and element.domain:
+            domain_classes.add(element.domain)
+            if include_ancestors:
+                slot_domain.extend(self.get_ancestors(element.domain, reflexive=True, mixin=mixin))
+            else:
+                slot_domain.append(element.domain)
+        for d in element.domain_of:
+            if d not in domain_classes:
                 if include_ancestors:
-                    slot_domain.extend(self.get_ancestors(element.domain, reflexive=True))
+                    slot_domain.extend(self.get_ancestors(d, reflexive=True, mixin=mixin))
                 else:
-                    slot_domain.append(element.domain)
-            for d in element.domain_of:
-                if d not in domain_classes:
-                    if include_ancestors:
-                        slot_domain.extend(self.get_ancestors(d, reflexive=True))
-                    else:
-                        slot_domain.append(d)
+                    slot_domain.append(d)
         return self._format_all_elements(slot_domain, formatted)
 
-    def get_slot_range(self, slot_name, include_ancestors: bool = False, formatted: bool = False) -> List[str]:
+    def get_slot_range(self, slot_name,
+                       include_ancestors: bool = False,
+                       formatted: bool = False,
+                       mixin: bool = True) -> List[str]:
         """
         Get the range for a given slot.
 
@@ -441,6 +477,9 @@ class Toolkit(object):
             Whether or not to include ancestors of the range class
         formatted: bool
             Whether to format element names as CURIEs
+        mixin: bool
+            If True, then that means we want to find mixin ancestors as well as is_a ancestors
+
 
         Returns
         -------
@@ -448,17 +487,19 @@ class Toolkit(object):
             The range for a given slot
 
         """
-        range = []
+        slot_range = []
         element = self.get_element(slot_name)
-        if element:
-            if element.range:
-                range.append(element.range)
+        if element and element.range:
+            slot_range.append(element.range)
             if include_ancestors:
-                ancs = self.get_ancestors(element.range, reflexive=False)
-                range.extend(ancs)
-        return self._format_all_elements(range, formatted)
+                ancs = self.get_ancestors(element.range, reflexive=False, mixin=mixin)
+                slot_range.extend(ancs)
+        return self._format_all_elements(slot_range, formatted)
 
-    def get_all_slots_with_class_domain(self, class_name, check_ancestors: bool = False, formatted: bool = False) -> List[str]:
+    def get_all_slots_with_class_domain(self, class_name,
+                                        check_ancestors: bool = False,
+                                        formatted: bool = False,
+                                        mixin: bool = True) -> List[str]:
         """
         Given a class, get all the slots where the class is the domain.
 
@@ -470,6 +511,8 @@ class Toolkit(object):
             Whether or not to lookup slots that include ancestors of the given class as its domain
         formatted: bool
             Whether to format element names as CURIEs
+        mixin: bool
+            If True, then that means we want to find mixin ancestors as well as is_a ancestors
 
         Returns
         -------
@@ -478,11 +521,14 @@ class Toolkit(object):
 
         """
         element = self.get_element(class_name)
-        slots = self._get_all_slots_with_class_domain(element, check_ancestors)
+        slots = self._get_all_slots_with_class_domain(element, check_ancestors, mixin)
         slot_names = [x.name for x in slots]
         return self._format_all_elements(slot_names, formatted)
 
-    def get_all_slots_with_class_range(self, class_name, check_ancestors: bool = False, formatted: bool = False) -> List[str]:
+    def get_all_slots_with_class_range(self, class_name,
+                                       check_ancestors: bool = False,
+                                       formatted: bool = False,
+                                       mixin: bool = True) -> List[str]:
         """
         Given a class, get all the slots where the class is the range.
 
@@ -494,6 +540,8 @@ class Toolkit(object):
             Whether or not to lookup slots that include ancestors of the given class as its range
         formatted: bool
             Whether to format element names as CURIEs
+        mixin: bool
+            If True, then that means we want to find mixin ancestors as well as is_a ancestors
 
         Returns
         -------
@@ -502,11 +550,14 @@ class Toolkit(object):
 
         """
         element = self.get_element(class_name)
-        slots = self._get_all_slots_with_class_range(element, check_ancestors)
+        slots = self._get_all_slots_with_class_range(element, check_ancestors, mixin)
         slot_names = [x.name for x in slots]
         return self._format_all_elements(slot_names, formatted)
 
-    def get_all_predicates_with_class_domain(self, class_name, check_ancestors: bool = False, formatted: bool = False) -> List[str]:
+    def get_all_predicates_with_class_domain(self, class_name,
+                                             check_ancestors: bool = False,
+                                             formatted: bool = False,
+                                             mixin: bool = True) -> List[str]:
         """
         Given a class, get all Biolink predicates where the class is the domain.
 
@@ -518,6 +569,8 @@ class Toolkit(object):
             Whether or not to lookup slots that include ancestors of the given class as its domain
         formatted: bool
             Whether to format element names as CURIEs
+        mixin: bool
+            If True, then that means we want to find mixin ancestors as well as is_a ancestors
 
         Returns
         -------
@@ -528,13 +581,16 @@ class Toolkit(object):
         filtered_slots = []
         element = self.get_element(class_name)
         if element:
-            slots = self._get_all_slots_with_class_domain(element, check_ancestors)
+            slots = self._get_all_slots_with_class_domain(element, check_ancestors, mixin)
             for s in slots:
-                if not s.alias and 'related to' in self.get_ancestors(s.name):
+                if not s.alias and RELATED_TO in self.get_ancestors(s.name, mixin):
                     filtered_slots.append(s.name)
         return self._format_all_elements(filtered_slots, formatted)
 
-    def get_all_predicates_with_class_range(self, class_name, check_ancestors: bool = False, formatted: bool = False):
+    def get_all_predicates_with_class_range(self, class_name,
+                                            check_ancestors: bool = False,
+                                            formatted: bool = False,
+                                            mixin: bool = True):
         """
         Given a class, get all Biolink predicates where the class is the range.
 
@@ -546,6 +602,8 @@ class Toolkit(object):
             Whether or not to lookup slots that include ancestors of the given class as its range
         formatted: bool
             Whether to format element names as CURIEs
+        mixin: bool
+            If True, then that means we want to find mixin ancestors as well as is_a ancestors
 
         Returns
         -------
@@ -556,13 +614,16 @@ class Toolkit(object):
         filtered_slots = []
         element = self.get_element(class_name)
         if element:
-            slots = self._get_all_slots_with_class_range(element, check_ancestors)
+            slots = self._get_all_slots_with_class_range(element, check_ancestors, mixin)
             for s in slots:
-                if not s.alias and 'related to' in self.get_ancestors(s.name):
+                if not s.alias and RELATED_TO in self.get_ancestors(s.name, mixin):
                     filtered_slots.append(s.name)
         return self._format_all_elements(filtered_slots, formatted)
 
-    def get_all_properties_with_class_domain(self, class_name, check_ancestors: bool = False, formatted: bool = False) -> List[str]:
+    def get_all_properties_with_class_domain(self, class_name,
+                                             check_ancestors: bool = False,
+                                             formatted: bool = False,
+                                             mixin: bool = True) -> List[str]:
         """
         Given a class, get all Biolink properties where the class is the domain.
 
@@ -574,6 +635,8 @@ class Toolkit(object):
             Whether or not to lookup slots that include ancestors of the given class as its domain
         formatted: bool
             Whether to format element names as CURIEs
+        mixin: bool
+            If True, then that means we want to find mixin ancestors as well as is_a ancestors
 
         Returns
         -------
@@ -584,13 +647,16 @@ class Toolkit(object):
         filtered_slots = []
         element = self.get_element(class_name)
         if element:
-            slots = self._get_all_slots_with_class_domain(element, check_ancestors)
+            slots = self._get_all_slots_with_class_domain(element, check_ancestors, mixin)
             for s in slots:
-                if not s.alias and 'related to' not in self.get_ancestors(s.name):
+                if not s.alias and RELATED_TO not in self.get_ancestors(s.name, mixin):
                     filtered_slots.append(s.name)
         return self._format_all_elements(filtered_slots, formatted)
 
-    def get_all_properties_with_class_range(self, class_name, check_ancestors: bool = False, formatted: bool = False) -> List[str]:
+    def get_all_properties_with_class_range(self, class_name,
+                                            check_ancestors: bool = False,
+                                            formatted: bool = False,
+                                            mixin: bool = True) -> List[str]:
         """
         Given a class, get all Biolink properties where the class is the range.
 
@@ -602,6 +668,8 @@ class Toolkit(object):
             Whether or not to lookup slots that include ancestors of the given class as its range
         formatted: bool
             Whether to format element names as CURIEs
+        mixin: bool
+            If True, then that means we want to find mixin ancestors as well as is_a ancestors
 
         Returns
         -------
@@ -612,13 +680,14 @@ class Toolkit(object):
         filtered_slots = []
         element = self.get_element(class_name)
         if element:
-            slots = self._get_all_slots_with_class_range(element, check_ancestors)
+            slots = self._get_all_slots_with_class_range(element, check_ancestors, mixin)
             for s in slots:
-                if not s.alias and 'related to' not in self.get_ancestors(s.name):
+                if not s.alias and RELATED_TO not in self.get_ancestors(s.name, mixin):
                     filtered_slots.append(s.name)
         return self._format_all_elements(filtered_slots, formatted)
 
-    def get_value_type_for_slot(self, slot_name, formatted: bool = False) -> str:
+    def get_value_type_for_slot(self, slot_name,
+                                formatted: bool = False) -> str:
         """
         Get the value type for a given slot.
 
@@ -649,7 +718,9 @@ class Toolkit(object):
                 element_type = et
         return element_type
 
-    def _get_all_slots_with_class_domain(self, element: Element, check_ancestors: bool) -> List[Element]:
+    def _get_all_slots_with_class_domain(self, element: Element,
+                                         check_ancestors: bool,
+                                         mixin: bool = True) -> List[Element]:
         """
         Given a class, get all the slots where the class is the domain.
 
@@ -659,6 +730,8 @@ class Toolkit(object):
             An element
         check_ancestors: bool
             Whether or not to lookup slots that include ancestors of the given class as its domain
+        mixin:
+            If True, then that means we want to find mixin ancestors as well as is_a ancestors
 
         Returns
         -------
@@ -669,15 +742,18 @@ class Toolkit(object):
         slots = []
         for k, v in self.generator.schema.slots.items():
             if check_ancestors:
-                if v.domain == element.name or v.domain in self.get_ancestors(element.name) \
-                        or element.name in v.domain_of or any(v.domain_of) in self.get_ancestors(element.name):
+                if v.domain == element.name or v.domain in self.get_ancestors(element.name, mixin) \
+                        or element.name in v.domain_of \
+                        or any(v.domain_of) in self.get_ancestors(element.name, mixin):
                     slots.append(v)
             else:
                 if element.name == v.domain or element.name in v.domain_of:
                     slots.append(v)
         return slots
 
-    def _get_all_slots_with_class_range(self, element: Element, check_ancestors: bool) -> List[Element]:
+    def _get_all_slots_with_class_range(self, element: Element,
+                                        check_ancestors: bool,
+                                        mixin: bool = True) -> List[Element]:
         """
         Given a class, get all the slots where the class is the range.
 
@@ -687,6 +763,8 @@ class Toolkit(object):
             An element
         check_ancestors: bool
             Whether or not to lookup slots that include ancestors of the given class as its range
+        mixin: bool
+            If True, then that means we want to find mixin ancestors as well as is_a ancestors
 
         Returns
         -------
@@ -697,32 +775,60 @@ class Toolkit(object):
         slots = []
         for k, v in self.generator.schema.slots.items():
             if check_ancestors:
-                if v.range == element.name or v.range in self.get_ancestors(element.name):
+                if v.range == element.name or v.range in self.get_ancestors(element.name, mixin):
                     slots.append(v)
             else:
-                if v.range:
-                    if element.name == v.range:
-                        slots.append(v)
+                if v.range and element.name == v.range:
+                    slots.append(v)
         return slots
 
     @lru_cache(CACHE_SIZE)
-    def is_predicate(self, name: str) -> bool:
+    def is_predicate(self, name: str, mixin: bool = True) -> bool:
         """
         Determines whether the given name is the name of an relation/predicate
         in the Biolink Model. An element is a predicate if it descends from
-        `related to`
+        `RELATED_TO`
 
         Parameters
         ----------
         name: str
             The name or alias of an element in the Biolink Model
+        mixin: bool
+            If True, then that means we want to find mixin ancestors as well as is_a ancestors
 
         Returns
         -------
         bool
             That the named element is a valid relation/predicate in Biolink Model
         """
-        return 'related to' in self.get_ancestors(name)
+        return RELATED_TO in self.get_ancestors(name, mixin)
+
+    @lru_cache(CACHE_SIZE)
+    def is_translator_canonical_predicate(self, name: str, mixin: bool = True) -> bool:
+        """
+        Determines whether the given name is the name of a canonical relation/predicate
+        in the Biolink Model. An element is a canonical predicate if it descends from
+        `RELATED_TO` and is tagged with the annotation 'biolink:canonical_predicate'
+
+        Parameters
+        ----------
+        name: str
+            The name or alias of an element in the Biolink Model
+        mixin: bool
+            If True, then that means we want to find mixin ancestors as well as is_a ancestors
+
+        Returns
+        -------
+        bool
+            That the named element is a valid translator canonical prediacte in Biolink Model
+        """
+        element = self.get_element(name)
+        annotation_tags = []
+        if element:
+            for annotation in element.annotations:
+                annotation_tags.append(annotation)
+        is_canonical = True if element is not None and 'biolink:canonical_predicate' in annotation_tags else False
+        return True if RELATED_TO in self.get_ancestors(name, mixin) and is_canonical else False
 
     @lru_cache(CACHE_SIZE)
     def is_mixin(self, name: str) -> bool:
@@ -749,7 +855,7 @@ class Toolkit(object):
         """
         Determines whether the given name is a predicate and if that predicate has an inverse defined
         in the Biolink Model. An element is a predicate if it descends from
-        `related to`
+        `RELATED_TO`
 
         Parameters
         ----------
@@ -789,7 +895,7 @@ class Toolkit(object):
         return subset in element.in_subset
 
     @lru_cache(CACHE_SIZE)
-    def is_category(self, name: str) -> bool:
+    def is_category(self, name: str, mixin: bool = True) -> bool:
         """
         Determines whether the given name is the name of a category in the
         Biolink Model. An element is a category if it descends from
@@ -799,17 +905,21 @@ class Toolkit(object):
         ----------
         name : str
             The name or alias of an element in the Biolink Model
+        mixin: bool
+            If True, then that means we want to find mixin ancestors as well as is_a ancestors
 
         Returns
         -------
         bool
             That the named element is a valid category in Biolink Model
-
         """
-        return 'named thing' in self.get_ancestors(name)
+        return 'named thing' in self.get_ancestors(name, mixin)
 
     @lru_cache(CACHE_SIZE)
-    def get_element_by_mapping(self, identifier: str, most_specific: bool = False, formatted: bool = False) -> Optional[str]:
+    def get_element_by_mapping(self, identifier: str,
+                               most_specific: bool = False,
+                               formatted: bool = False,
+                               mixin: bool = True) -> Optional[str]:
         """
         Get a Biolink Model element by mapping.
         This method return the common ancestor of the set of elements referenced by uriorcurie.
@@ -823,6 +933,8 @@ class Toolkit(object):
             or to get all mappings of varying specificity
         formatted: bool
             Whether to format element names as CURIEs
+        mixin: bool
+            If True, then that means we want to find mixin ancestors as well as is_a ancestors
 
         Returns
         -------
@@ -837,7 +949,7 @@ class Toolkit(object):
         if mappings:
             ancestors: List[List[str]] = []
             for m in mappings:
-                ancestors.append([x for x in self.get_ancestors(m)[::-1] if x in mappings])
+                ancestors.append([x for x in self.get_ancestors(m, mixin)[::-1] if x in mappings])
             common_ancestors = reduce(lambda s, l: s.intersection(set(l)), ancestors[1:], set(ancestors[0]))
             for a in ancestors[0]:
                 if a in common_ancestors:
@@ -1050,29 +1162,44 @@ class Toolkit(object):
             formatted_elements = elements
         return formatted_elements
 
+    @lru_cache(CACHE_SIZE)
+    def get_model_version(self) -> str:
+        """
+        Return the version of the biolink-model in use.
+
+        Returns
+        -------
+        str
+            The biolink-model version
+
+        """
+        return self.generator.schema.version
+
+
+
     @deprecation.deprecated(deprecated_in='0.3.0', removed_in='1.0', details='Use get_all_elements method instead')
     def names(self, formatted: bool = False) -> List[str]:
         return self.get_all_elements(formatted)
 
     @deprecation.deprecated(deprecated_in='0.2.0', removed_in='1.0', details='Use get_descendants method instead')
-    def descendents(self, name: str) -> List[str]:
-        return self.get_descendants(name)
+    def descendents(self, name: str, mixin: bool = True) -> List[str]:
+        return self.get_descendants(name, mixin)
 
     @deprecation.deprecated(deprecated_in='0.2.0', removed_in='1.0', details='Use get_ancestors method instead')
-    def ancestors(self, name: str) -> List[str]:
-        return self.get_ancestors(name)
+    def ancestors(self, name: str, mixin: bool = True) -> List[str]:
+        return self.get_ancestors(name, mixin)
 
     @deprecation.deprecated(deprecated_in='0.2.0', removed_in='1.0', details='Use get_children method instead')
-    def children(self, name: str) -> List[str]:
-        return self.get_children(name)
+    def children(self, name: str, mixin: bool = True) -> List[str]:
+        return self.get_children(name, mixin)
 
     @deprecation.deprecated(deprecated_in='0.2.0', removed_in='1.0', details='Use get_parent method instead')
-    def parent(self, name: str) -> Optional[str]:
-        return self.get_parent(name)
+    def parent(self, name: str, mixin: bool = True) -> Optional[str]:
+        return self.get_parent(name, mixin)
 
     @deprecation.deprecated(deprecated_in='0.1.1', removed_in='1.0', details='Use is_predicate method instead')
-    def is_edgelabel(self, name: str) -> bool:
-        return self.is_predicate(name)
+    def is_edgelabel(self, name: str, mixin: bool = True) -> bool:
+        return self.is_predicate(name, mixin)
 
     @deprecation.deprecated(deprecated_in='0.1.1', removed_in='1.0', details='Use get_all_elements_by_mapping method instead')
     def get_all_by_mapping(self, uriorcurie: str) -> List[str]:
